local app = application

local function map(v, minSrc, maxSrc, minDst, maxDst)
	return (v - minSrc) / (maxSrc - minSrc) * (maxDst - minDst) + minDst
end

FPSgraph = Core.class(Sprite)

function FPSgraph:init(width, height, font, color, alpha)
	color = color or 0xffffff
	alpha = alpha or 0.5
	font = font or Font.getDefault()
	
	local maxTextHeight = font:getLineHeight() * 2
	self.meshHeight = (height - maxTextHeight)
	
	self.w = width
	self.h = height
	
	self.bg = Pixel.new(color, alpha, width, height)
	self:addChild(self.bg)
	
	self.mesh = Mesh.new()
	self.mesh:setY(maxTextHeight)
	self:addChild(self.mesh)
	
	self.fpsTF = TextField.new(f, "")
	self.fpsTF:setLayout{
		w = width, h = maxTextHeight,
		flags = 1280 | FontBase.TLF_CENTER | FontBase.TLF_VCENTER
	}
	self:addChild(self.fpsTF)
	
	self.timer = 0
	
	self.gap = 10
	
	self.maxFps = 0
	self.minFps = 100000
	self.updateTime = 0.2
	self.data = {}
	
	-- ignore application scale mode
	self:addEventListener("applicationResize", self.onAppResize, self)
	self:onAppResize()
end
--
function FPSgraph:onAppResize()
	local dx = app:getLogicalScaleX()
	local dy = app:getLogicalScaleY()
	local tx = app:getLogicalTranslateX()
	local ty = app:getLogicalTranslateY()
	self:setPosition(-tx / dx, -ty / dy)
	self:setScale(1/app:getLogicalScaleX(), 1/app:getLogicalScaleY())
end
--
function FPSgraph:updateMesh()
	local prevMax = self.maxFps
	self.maxFps = 0
	self.minFps = 100000
	
	local maxFps = app:getFps()
	local m = self.mesh
	local h = self.meshHeight
	local j = 1
	local vi = 1
	local n = #self.data
	for i = 1, n - 1 do
		local fps = self.data[i]
		
		local y = map(fps, 0, prevMax, h, 0)
		
		local x = (i - 1) * self.gap
		m:setVertex(vi + 0, x, y)
		m:setVertex(vi + 1, x, h)
		
		m:setIndex(j + 0, vi + 0)
		m:setIndex(j + 1, vi + 1)
		m:setIndex(j + 2, vi + 2)
		m:setIndex(j + 3, vi + 1)
		m:setIndex(j + 4, vi + 2)
		m:setIndex(j + 5, vi + 3)
		
		m:setColor(vi + 0, 0x00ff00, 1)
		m:setColor(vi + 1, 0xff0000, 1)
		
		
		self.maxFps = self.maxFps <> fps
		self.minFps = self.minFps >< fps
		
		j += 6
		vi += 2
	end
	local fps = self.data[n]
	local x = (n - 1) * self.gap
	local y = map(fps, 0, prevMax, h, 0)
	
	m:setVertex(n * 2 + 0, x, y)
	m:setVertex(n * 2 + 1, x, h)
	
	self.maxFps = self.maxFps <> fps
	self.minFps = self.minFps >< fps
end
--
function FPSgraph:update(dt)
	self.timer += dt
	
	if (self.timer > self.updateTime) then
		self.timer = 0
		local fps = 1 / dt
		
		local n = #self.data
		if (n < (self.w + self.gap * 2) // self.gap) then 
			self.data[n+1] = fps
		else
			table.remove(self.data, 1)
			self.data[n] = fps
		end
		
		self:updateMesh()
		
		self.fpsTF:setText(("Min: %03i  Curr: %03i  Max: %03i"):format(self.minFps // 1, fps, self.maxFps // 1))
		
	end
end